from timeit import timeit
from timeit import repeat
import numpy as np
import os
import sys
import math

def readResultsReport(fileName):
	with open(fileName,'r') as report:
		for line in report:
			if "total_time" in line:
				total_time = line[12:]
	return float(total_time)
	
def readOutput(fileName):
	toReturn = dict()
	with open(fileName,'r') as report:
		for line in report:
			if "   - all: " in line:
				toReturn['all'] = line[10:]
			elif "   - killed: " in line:
				toReturn['killed'] = line[13:]
			elif "   - survived: " in line:
				toReturn['survived'] = line[15:]
			elif "   - incompetent: " in line:
				toReturn['incompetent'] = line[18:]
			elif "   - timeout: " in line:
				toReturn['timeout'] = line[14:]
	if not 'all' in toReturn.keys():
		toReturn['all'] = '0'
	if not 'killed' in toReturn.keys():
		toReturn['killed'] = '0'
	if not 'survived' in toReturn.keys():
		toReturn['survived'] = '0'
	if not 'incompetent' in toReturn.keys():
		toReturn['incompetent'] = '0'
	if not 'timeout' in toReturn.keys():
		toReturn['timeout'] = '0'
	return toReturn
	
class testResult:
	def __init__(self,time=0,mutantDictionary=None):
		self.time = time
		self.Mall = 0
		self.Mkilled = 0
		self.Msurvived = 0
		self.Mincompetent = 0
		self.Mtimeout = 0
		if mutantDictionary is not None:
			self.analyzeMutantDictionary(mutantDictionary)
	def __str__(self):
		st = "total time:" +str(self.time)
		st += "\n total mutants:" +str(self.Mall)
		st += "\n\t killed mutants:" +str(self.Mkilled)
		st += "\n\t survived mutants:" +str(self.Msurvived)
		st += "\n\t incompetent mutants:" +str(self.Mincompetent)
		st += "\n\t timeout mutants:" +str(self.Mtimeout)
		return st
	def analyzeMutantDictionary(self,mDict):
		words = mDict['all'].split(' ')
		self.Mall = int(words[0])
		words = mDict['killed'].split(' ')
		self.Mkilled = int(words[0])
		words = mDict['survived'].split(' ')
		self.Msurvived = int(words[0])
		words = mDict['incompetent'].split(' ')
		self.Mincompetent = int(words[0])
		words = mDict['timeout'].split(' ')
		self.Mtimeout = int(words[0])
		return mDict

def conductTest(resultList,cargs,call_setup,repetitions,reportFileName,silent=False):
	if not silent:
		print("call_arguments:"+cargs[0]+"\n")
	list_of_time_results = []
	list_of_results = []
	for x in range(0,repetitions):
		if not silent:
			print('{0} of {1}'.format(x,repetitions),end="\r")
		time_taken = timeit(stmt = cargs[0],setup = call_setup,number = 1)
		list_of_time_results.append(time_taken)
		a = readResultsReport(reportFileName)
		b = readOutput("outputs.txt")
		tr = testResult(time=a,mutantDictionary=b)
		list_of_results.append(tr)
	
	average = 0
	list_of_mutpy_time_results = []
	for value in list_of_results:
		average+=value.time
		list_of_mutpy_time_results.append(value.time)
	average = average/len(list_of_results)
	
	a = readResultsReport(reportFileName)
	b = readOutput("outputs.txt")
	tr = testResult(time=a,mutantDictionary=b)
	resultList[cargs[1]] = tr
	if not silent:
		print("MutPy time taken: "+str(tr.time))
		print("MutPy average time taken: "+str(average))
		print("MutPy average time taken2: "+str(np.average(list_of_mutpy_time_results)))
		print("MutPy mean: "+str(np.mean(list_of_mutpy_time_results)))
		print("MutPy variation: "+str(np.var(list_of_mutpy_time_results,ddof=1)))
		print("Average time timeit: "+str(np.average(list_of_time_results)))
		print("Mean from timeit: "+str(np.mean(list_of_time_results)))
		print("Variation in timeit: "+str(np.var(list_of_time_results,ddof=1)))
		print(tr.__str__())
		print('*******************************')
	return average
	
def oldTests(test_data):
	test_data['call_arguments_list'] = call_arguments_list
	test_data['call_arguments_list_no_type_check'] = call_arguments_list_no_type_check
	type_check_scores = test_data['type_check_scores']
	no_type_check_scores = test_data['no_type_check_scores']
	call_setup = test_data['call_setup']
	repetitions = test_data['repetitions']
	reportFileName = test_data['reportFileName']
	for cargs in call_arguments_list:
		print('type check')
		conductTest(type_check_scores,cargs,call_setup,repetitions,reportFileName)

	for cargs in call_arguments_list_no_type_check:
		print('no type check')
		conductTest(no_type_check_scores,cargs,call_setup,repetitions,reportFileName)
	
def newTests(test_data):	
	test_data['call_arguments_list'] = call_arguments_list
	test_data['call_arguments_list_no_type_check'] = call_arguments_list_no_type_check
	type_check_scores = test_data['type_check_scores']
	no_type_check_scores = test_data['no_type_check_scores']
	call_setup = test_data['call_setup']
	repetitions = test_data['repetitions']
	reportFileName = test_data['reportFileName']
	type_check_aver = {}
	no_type_check_aver = {}

	for cargs in call_arguments_list:
		type_check_aver[cargs[1]] = []
	for cargs in call_arguments_list_no_type_check:
		no_type_check_aver[cargs[1]] = []
	for x in range(0,repetitions):
		print('{0} of {1}'.format(x,repetitions),end="")
		for cargs in call_arguments_list:
			type_check_aver[cargs[1]].append(conductTest(type_check_scores,cargs,call_setup,1,reportFileName,True))
			print('.',end="")
		print (' (check ready)',end="")
		for cargs in call_arguments_list_no_type_check:	
			no_type_check_aver[cargs[1]].append(conductTest(no_type_check_scores,cargs,call_setup,1,reportFileName,True))
			print('.',end="")
		print(' (no check)',end='\r')
	
	def calculate_s_deviation(list,aver):
		sum = 0
		for value in list:
			sum+=value*value
		sum /= len(list)
		sum -= aver*aver
		return math.sqrt(sum)
	def print_carg_avers(cargs,avers):
		sum = 0
		for x in avers[cargs[1]]:
			sum+=x
		average = sum/len(avers[cargs[1]])
		print('\n average time for '+cargs[1])
		print('is: {}'.format(average))
		print("standard deviation:{}".format(calculate_s_deviation(avers[cargs[1]],average)))
		
	for cargs in call_arguments_list:
		print("\n\t(CHECK)",end="")
		print_carg_avers(cargs,type_check_aver)
		print(type_check_scores[cargs[1]])
	for cargs in call_arguments_list_no_type_check:
		print("\n\t(NO CHECK)",end="")
		print_carg_avers(cargs,no_type_check_aver)
		print(no_type_check_scores[cargs[1]])
	
	
counter = 0
for x in reversed(os.path.abspath(sys.argv[0])):
	if x=='\\':
		break
	counter+=1
documentsPath = os.path.abspath(sys.argv[0])[0:-counter]


#target_test_tuple_list = [('laczenie02','laczenie02_test')]
target_test_tuple_list = [('find_classes','find_classes_test'),('annotation01','annotation01_test'),('annotation02','annotation02_test'),('laczenie01','laczenie01_test'),('AdvantageCalculations','AdvantageCalculations_test'),('laczenie02','laczenie02_test')]


repetitions = 2
if len(sys.argv) > 1:
	try:
		repetitions = int(sys.argv[1])
	except ValueError:
		print('couldn\'t parse argument into integer. Using default value.')
else:
	print('using default value.')
print("repetition value is:{}".format(repetitions))
stdoutFile = open("outputs.txt",'w')
stderrFile = open("errors.txt",'w')
reportFileName = 'rep.txt'
stdoutFile.close()
stderrFile.close()

call_arguments_list = []
call_arguments_list_no_type_check = []
for t in target_test_tuple_list:
	external_command = "mut.py -m --target "+t[0]+" --unit-test "+t[1]+" -r "+reportFileName+" --type-check --coverage"
	call_arguments_list.append(("""subprocess.run('{0}',shell=True,stdout=stdoutFile,stderr=stderrFile)""".format(external_command),t[0]))
	external_command = "mut.py -m --target "+t[0]+" --unit-test "+t[1]+" -r "+reportFileName+" --coverage"
	call_arguments_list_no_type_check.append(("""subprocess.run('{0}',shell=True,stdout=stdoutFile,stderr=stderrFile)""".format(external_command),t[0]))
call_arguments = """subprocess.run('{0}',shell=True,stdout=stdoutFile,stderr=stderrFile)""".format(external_command)

call_setup = """import subprocess\n\
stdoutFile = open("outputs.txt",'w')\n\
stderrFile = open("errors.txt",'w')"""

type_check_scores = {}
no_type_check_scores = {}

test_data = dict()
test_data['call_arguments_list'] = call_arguments_list
test_data['call_arguments_list_no_type_check'] = call_arguments_list_no_type_check
test_data['type_check_scores'] = type_check_scores
test_data['no_type_check_scores'] = no_type_check_scores
test_data['call_setup'] = call_setup
test_data['repetitions'] = repetitions
test_data['reportFileName'] = reportFileName

newTests(test_data)

